
subroutine CalcEnergy(Pos, Lat, NLat, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: NLat, M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NLat-1, 0:Dim-1) :: Lat
    real(8), intent(in), dimension(Dim) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1., sigP = 1.5, sigL = 1.8
    real(8), dimension(Dim) :: rij, ril, Fij, Posi
    real(8) :: d, d2, id2, id6, id12, dl2, idl2, idl6, idl12
    real(8) :: rc2, Shift, ShiftL
    integer :: i, j, z
    PEnergy = 0. 
    Shift = -4. * ((rc / sigP)**(-12) - (rc / sigP)**(-6)) 
    ShiftL = -4. * ((rc / sigL)**(-12) - (rc / sigL)**(-6))
    rc2 = rc * rc 
    do i = 0, NAtom - 1 
        !store Pos(i,:) in a temporary array for faster access in j loop 
        Posi = Pos(i,:) 

        do j = i + 1, (NAtom - 1)
            if (j == i + 1 .and. mod(j, M) > 0) then 
                !this is a bonded interaction
                rij = Pos(j,:) - Posi 
                rij = rij - L * dnint(rij / L)
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0)**2
            else 
                !this is a nonbonded interaction- use LJ potential
                !compute squared distance form atom i to atom j
                rij = Pos(j,:) - Posi 
                rij = rij - L * dnint(rij / L)
                d2 = sum(rij * rij)
                !check to see if rij^2 (atom i to j) > cutoff^2
                if (d2 > rc2) then 	
                    cycle
                endif
                id2 = sigP * sigP / d2            !inverse squared distance 
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift
            endif
        enddo 
		
        !loop through non-bonded interactions between atom i and lattice
        do z = 0, NLat - 1
            !compute squared distance from atom i to lattice
            ril = Lat(z,:) - Posi
            ril = ril - L * dnint(rij / L)
            dl2 = sum(ril * ril)
            !check to see if ril^2 (atom i to lattice) > cutoff^2
            if (dl2 > rc2) then 
                cycle
            endif
            idl2 = sigL * sigL / dl2
            idl6 = idl2 * idl2 * idl2
            idl12 = idl6 * idl6
            PEnergy = PEnergy + 4. * (idl12 - idl6) + ShiftL
        enddo
    enddo  
end subroutine


subroutine CalcEnergyForces(Pos, Lat, NLat, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: NLat, M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NLat-1, 0:Dim-1) :: Lat
    real(8), intent(in), dimension(Dim) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1., sigP = 1.5, sigL = 1.8
    real(8), dimension(Dim) :: rij, ril, Fij, Fil, Posi 
    real(8) :: d, d2, id2, id6, id12, dl2, idl2, idl6, idl12
    real(8) :: rc2, Shift, ShiftL
    integer :: i, j, z
    PEnergy = 0. 
    Forces = 0. 
    Shift = -4. * ((rc / sigP)**(-12) - (rc / sigP)**(-6))
    ShiftL = -4. * ((rc / sigL)**(-12) - (rc / sigL)**(-6))

    rc2 = rc * rc 
    do i = 0, NAtom - 1 
        !store Pos(i,:) in a temporary array for faster access in j loop 
        Posi = Pos(i,:) 

        do j = i + 1, NAtom - 1
            if (j == i + 1 .and. mod(j, M) > 0) then 
            !this is a bonded interaction
                rij = Pos(j,:) - Posi 
                rij = rij - L * dnint(rij / L)
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0)**2
                Fij = rij * k * (d - r0) * (1./d)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij 
            else 
            !this is a nonbonded interaction- use LJ potential
            	!compute only the squared distance and compare to squared cut 
                rij = Pos(j,:) - Posi 
                rij = rij - L * dnint(rij / L)
                d2 = sum(rij * rij) 
                if (d2 > rc2) then 	
                    cycle
                endif 
                id2 = sigP * sigP / d2            !inverse squared distance 
                id6 = id2 * id2 * id2    !inverse sixth distance 
                id12 = id6 * id6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift 
                Fij = rij * ((-48. * id12 + 24. * id6) * id2) 
                Forces(i,:) = Forces(i,:) + Fij 
                Forces(j,:) = Forces(j,:) - Fij 
            endif
        enddo 
		
	!loop through non-bonded interactions between atom i and lattice
        do z = 0, NLat - 1
	    !compute squared distance from atom i to lattice
            ril = Lat(z,:) - Posi
	    ril = ril - L * dnint(rij / L)
            dl2 = sum(ril * ril)
	    !check to see if ril^2 (atom i to lattice) > cutoff^2
            if (dl2 > rc2) then 
                cycle
            endif
            idl2 = sigL * sigL / dl2
            idl6 = idl2 * idl2 * idl2
            idl12 = idl6 * idl6
            PEnergy = PEnergy + 4. * (idl12 - idl6) + ShiftL
            Fil = ril * ((-48. * idl12 + 24. * idl6) * idl2) 
            Forces(i,:) = Forces(i,:) + Fil
        enddo
    enddo 
end subroutine


subroutine VVIntegrate(Pos, Lat, NLat, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: NLat, M, Dim, NAtom
    real(8), intent(in), dimension(0:NLat-1, 0:Dim-1) :: Lat
    real(8), intent(in), dimension(Dim) :: L
    real(8), intent(in) :: rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Lat, NLat, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine